package com.xkrs.service.impl;

import com.xkrs.common.encapsulation.PromptMessageEnum;
import com.xkrs.dao.FirePointOrdinaryDao;
import com.xkrs.dao.StreetDao;
import com.xkrs.dao.SysUserDao;
import com.xkrs.helper.FirePointPushManager;
import com.xkrs.helper.FirePointQueryManager;
import com.xkrs.model.bean.AddressBean;
import com.xkrs.model.bean.DataWrapper2;
import com.xkrs.model.entity.FirePointOrdinaryEntity;
import com.xkrs.model.entity.StreetEntity;
import com.xkrs.model.qo.AllFirePointQo;
import com.xkrs.model.qo.UpdateFirePointQo;
import com.xkrs.service.DispatchFirePointService;
import com.xkrs.utilsnew.DateTimeUtils;
import com.xkrs.utilsnew.ErrorInfoLogUtils;
import com.xkrs.utilsnew.FirePointAddressUtils;
import com.xkrs.utilsnew.FirePointFilterUtils;
import org.apache.hc.core5.util.TextUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.persistence.criteria.Predicate;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import static com.xkrs.common.encapsulation.OutputEncapsulation.outputEncapsulationObject;

@Service
public class DispatchFirePointServiceImpl implements DispatchFirePointService {

    public static Logger log = LoggerFactory.getLogger(DispatchFirePointServiceImpl.class);

    private final Locale locale = LocaleContextHolder.getLocale();

    @Resource
    private FirePointOrdinaryDao firePointOrdinaryDao;

    @Resource
    private FirePointQueryManager firePointQueryManager;

    @Resource
    private FirePointPushManager firePointPushManager;

    @Resource
    private SysUserDao sysUserDao;

    @Resource
    private StreetDao streetDao;

    /**
     * 推送火点到秸秆系统
     */
    @Override
    public String insertFirePoint(AllFirePointQo firePointQo) {
        DataWrapper2<Boolean, String> checkLandTypeWrapper = FirePointFilterUtils.checkLandType(firePointQo.getLandtype());
        if (!checkLandTypeWrapper.getData1()) {
            return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL, checkLandTypeWrapper.getData2(), locale);
        }
        DataWrapper2<Boolean, String> checkSatelliteTypeWrapper = FirePointFilterUtils.checkSatelliteType(firePointQo.getSatelliteType(), firePointQo.getConfidence());
        if (!checkSatelliteTypeWrapper.getData1()) {
            return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL, checkSatelliteTypeWrapper.getData2(), locale);
        }
        log.info("insertFirePoint 接收到火点信息：" + firePointQo.toString());
        //解构接收的数据
        String fireCode = firePointQo.getFireCode();
        Long satelliteTimeTs = firePointQo.getSatelliteTimeTs();
        Double longitude = firePointQo.getLongitude();
        Double latitude = firePointQo.getLatitude();
        String satelliteType = firePointQo.getSatelliteType();
        String landType = firePointQo.getLandtype();
        String confidence = firePointQo.getConfidence();
        String fireImage = firePointQo.getFireImage();
        String satelliteImage = firePointQo.getSatelliteImage();
        String countyCode = String.valueOf(firePointQo.getCountyCode());
        String countyName = firePointQo.getCountyName();
        String townCode = firePointQo.getTowncode();
        //格式化卫星时间和添加时间
        LocalDateTime satelliteLocalDateTime = LocalDateTime.ofEpochSecond(satelliteTimeTs, 0, ZoneOffset.ofHours(8));
        String satelliteTime = DateTimeUtils.localDateTimeToString(satelliteLocalDateTime);//卫星时间
        String addTime = DateTimeUtils.localDateTimeToString(LocalDateTime.now());//添加时间
        //生成实体类
        FirePointOrdinaryEntity firePointOrdinaryEntity = new FirePointOrdinaryEntity();
        firePointOrdinaryEntity.setFireCode(fireCode);
        firePointOrdinaryEntity.setFireStatus(null);//默认火点的研判标志是null：伪火点
        firePointOrdinaryEntity.setSatelliteTime(satelliteTime);
        firePointOrdinaryEntity.setLongitude(longitude);
        firePointOrdinaryEntity.setLatitude(latitude);
        firePointOrdinaryEntity.setSatelliteType(satelliteType);
        firePointOrdinaryEntity.setLandType(landType);
        firePointOrdinaryEntity.setConfidence(confidence);
        firePointOrdinaryEntity.setFireImage(fireImage);
        firePointOrdinaryEntity.setSatelliteImage(satelliteImage);
        firePointOrdinaryEntity.setAddTime(addTime);
        firePointOrdinaryEntity.setRemark("");
        //过滤重复数据
        if (FirePointFilterUtils.checkDuplicated(firePointOrdinaryDao, firePointOrdinaryEntity)) {
            return outputEncapsulationObject(PromptMessageEnum.DATA_EXIT, "该火点和历史火点重复", locale);
        }
        //使用网络方式解析地址信息
        AddressBean addressBean = FirePointAddressUtils.analysisWithNetwork(firePointOrdinaryEntity.getLongitude(), firePointOrdinaryEntity.getLatitude());
        if (addressBean.isSuccess()) {
            //使用网络方式解析地址信息成功，就对实体类对象赋值入库。
            firePointOrdinaryEntity.setProCode(addressBean.getProCode());
            firePointOrdinaryEntity.setProName(addressBean.getProName());
            firePointOrdinaryEntity.setCityCode(addressBean.getCityCode());
            firePointOrdinaryEntity.setCityName(addressBean.getCityName());
            firePointOrdinaryEntity.setCountyCode(addressBean.getCountyCode());
            firePointOrdinaryEntity.setCountyName(addressBean.getCountyName());
            firePointOrdinaryEntity.setTownCode(addressBean.getTownCode());
            firePointOrdinaryEntity.setTownName(addressBean.getTownName());
            firePointOrdinaryEntity.setFirePointAddress(addressBean.getFirePointAddress());
            firePointOrdinaryDao.save(firePointOrdinaryEntity);
            firePointPushManager.pushNotification(firePointOrdinaryEntity);//将火点消息通知用户
            return outputEncapsulationObject(PromptMessageEnum.SUCCESS, "添加成功", locale);
        }
        ErrorInfoLogUtils.log("高德逆地理编码异常，出现问题的火点详情：" + firePointOrdinaryEntity.toString());
        //使用网络方式解析地址信息失败，就使用接收到的数据结合字典表对实体类对象赋值入库。
        StreetEntity streetEntity = obtainInfoByCountyCode(countyCode);
        if (streetEntity != null) {
            firePointOrdinaryEntity.setProCode(streetEntity.getProCode());
            firePointOrdinaryEntity.setProName(streetEntity.getProName());
            firePointOrdinaryEntity.setCityCode(streetEntity.getCityCode());
            firePointOrdinaryEntity.setCityName(streetEntity.getCityName());
        }
        firePointOrdinaryEntity.setCountyCode(countyCode);
        firePointOrdinaryEntity.setCountyName(countyName);
        firePointOrdinaryEntity.setTownCode(townCode);
        firePointOrdinaryDao.save(firePointOrdinaryEntity);
        firePointPushManager.pushNotification(firePointOrdinaryEntity);//将火点消息通知用户
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS, "添加成功", locale);
    }

    private StreetEntity obtainInfoByCountyCode(String countyCode) {
        try {
            return streetDao.findByCountyCode(countyCode).get(0);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 更新火点状态
     */
    @Override
    public String updateFirePoint(UpdateFirePointQo updateFirePointQo) {
        List<UpdateFirePointQo.DataDTO> list = updateFirePointQo.getData();
        //先遍历一遍接收到的参数集合，参数不合法就返回错误，参数全部OK就进入下面的循环更新火点状态
        for (UpdateFirePointQo.DataDTO item : list) {
            String fireCode = item.getFireCode();
            String status = item.getStatus();
            if ((!TextUtils.isEmpty(fireCode)) && (!TextUtils.isEmpty(status)) && (("0".equals(status)) || ("1".equals(status)))) {
                System.out.println("参数正常：fireCode = " + fireCode + "，status = " + fireCode);
            } else {
                ErrorInfoLogUtils.log("updateFirePoint 参数异常：fireCode = " + fireCode + "，status = " + fireCode);
                return outputEncapsulationObject(PromptMessageEnum.PARAM_ILLEGAL, "更新失败", locale);
            }
        }
        //参数全部OK，更新火点状态
        for (UpdateFirePointQo.DataDTO item : list) {
            try {
                String fireCode = item.getFireCode();
                String status = item.getStatus();
                Specification<FirePointOrdinaryEntity> specification = (root, criteriaQuery, criteriaBuilder) -> {
                    //查询条件集合
                    List<Predicate> predicateList = new ArrayList<>();
                    predicateList.add(criteriaBuilder.equal(root.get("fireCode").as(String.class), fireCode));
                    Predicate[] predicateArray = new Predicate[predicateList.size()];
                    return criteriaBuilder.and(predicateList.toArray(predicateArray));
                };
                //根据fireCode找到火点
                List<FirePointOrdinaryEntity> firePointList = firePointOrdinaryDao.findAll(specification);
                for (FirePointOrdinaryEntity firePoint : firePointList) {
                    firePoint.setFireStatus(status);//设置火点的研判标志属性
                    firePointOrdinaryDao.save(firePoint);//数据库保存
                }
                if (firePointList.size() != 1) {
                    ErrorInfoLogUtils.log("item 更新成功。数据异常，审核系统发送的fireCode在秸秆系统中size() != 1：fireCode = " + fireCode + "，status = " + fireCode);
                }
            } catch (Exception e) {
                e.printStackTrace();
                return outputEncapsulationObject(PromptMessageEnum.PROCESS_FAIL, "item 更新失败", locale);
            }
        }
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS, "for 更新成功", locale);
    }

}
