package com.xkrs.utilsnew;

import org.apache.hc.core5.util.TextUtils;
import org.geotools.data.simple.SimpleFeatureCollection;
import org.geotools.data.simple.SimpleFeatureIterator;
import org.geotools.filter.text.cql2.CQL;
import org.geotools.filter.text.cql2.CQLException;
import org.locationtech.jts.geom.*;
import org.locationtech.jts.operation.buffer.BufferParameters;
import org.opengis.feature.simple.SimpleFeature;
import org.opengis.filter.Filter;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class BufferAnalysisUtils {

    private BufferAnalysisUtils() {
    }

    public static void main(String[] args) throws CQLException {
        String shapeFilePath = "/Users/liuchengqian/Desktop/data/shandong_land_type_2022.shp";
        SimpleFeatureCollection simpleFeatureCollection = ShpFileUtils.readShpFileFeatureCollection(shapeFilePath);
        double longitude = 116.218;//经度
        double latitude = 36.0852;//纬度
        double distance = getBufferDistance(1500L);// buffer 宽度 1.5公里
        GeometryFactory geometryFactory = new GeometryFactory(new PrecisionModel(PrecisionModel.FLOATING), 4326);
        Coordinate[] coordinates = new Coordinate[]{new Coordinate(longitude, latitude)};
        Point point = new Point(geometryFactory.getCoordinateSequenceFactory().create(coordinates), geometryFactory);
        Geometry pointBuffer = point.buffer(distance, 40, BufferParameters.CAP_ROUND);
        String pointBufferWkt = pointBuffer.toText();
        double pointBufferArea = pointBuffer.getArea();//总面积
        // 相交过滤
        Filter intersectsFilter = CQL.toFilter(String.format("INTERSECTS(the_geom, %s)", pointBufferWkt));
        SimpleFeatureCollection filterSimpleFeatureCollection = simpleFeatureCollection.subCollection(intersectsFilter);
        //地物类型对应的字段
        String landTypeProperty = "tblxmc";
        String defaultLandType = "其它";
        List<String> targetLandTypeList = new ArrayList<>();
        targetLandTypeList.add("林地");
        targetLandTypeList.add("农作物");
        targetLandTypeList.add("水域");
        targetLandTypeList.add("建筑物");
        targetLandTypeList.add("水产养殖设施农用地");
//        targetLandTypeList.add("其他土地");
        Map<String, Double> targetLandTypeAreaMap = new HashMap<>();
        for (String targetLandType : targetLandTypeList) {
            targetLandTypeAreaMap.put(targetLandType, 0D);
        }
        SimpleFeatureIterator featureIterator = filterSimpleFeatureCollection.features();
        while (featureIterator.hasNext()) {
            try {
                SimpleFeature simpleFeature = featureIterator.next();
                if (simpleFeature == null) {
                    System.out.println("simpleFeature == null");
                    continue;
                }
                Geometry geometry = (Geometry) simpleFeature.getDefaultGeometry();
                if (geometry == null) {
                    System.out.println("geometry == null");
                    continue;
                }
                String landType = simpleFeature.getProperty(landTypeProperty).getValue().toString().trim();
                if (TextUtils.isEmpty(landType) || (!targetLandTypeAreaMap.containsKey(landType))) {
                    System.out.println("!targetLandTypeMap.containsKey(landType)， landType = " + landType);
                    continue;
                }
                // 检查多边形是否存在自相交或重叠的情况，如果有则修复
                Geometry geometryBuffer = geometry.buffer(0D);
                if (!pointBuffer.intersects(geometryBuffer)) {
                    System.out.println("!pointBuffer.intersects(geometryBuffer)");
                    continue;
                }
                Geometry intersectGeometry = pointBuffer.intersection(geometryBuffer);
                if (intersectGeometry == null) {
                    System.out.println("intersectGeometry == null");
                    continue;
                }
                double intersectArea = intersectGeometry.getArea();
                if (intersectArea < 0D) {
                    System.out.println("intersectArea < 0");
                    continue;
                }
                targetLandTypeAreaMap.put(landType, targetLandTypeAreaMap.get(landType) + intersectArea);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        double targetLandAreaSum = 0D;
        StringBuilder builder = new StringBuilder();
        for (String targetLandType : targetLandTypeList) {
            double currentArea = targetLandTypeAreaMap.get(targetLandType);
            targetLandAreaSum += currentArea;
            builder.append(targetLandType);
            builder.append("=");
            builder.append(formatDoublePer(currentArea / pointBufferArea));
            builder.append(", ");
        }
        builder.append(defaultLandType);
        builder.append("=");
        builder.append(formatDoublePer((pointBufferArea - targetLandAreaSum) / pointBufferArea));
        String result = builder.toString();
        System.out.println(result);
    }


    public static final int RADIUS = 6377830;

    private static double getBufferDistance(long meter) {
        return meter * 180.D / Math.PI / RADIUS;
    }

    private static final DecimalFormat decimalFormat = new DecimalFormat("0.00");

    private static String formatDoublePer(double value) {
        return decimalFormat.format(Math.abs(value) * 100D) + "%";
    }

}
