package com.xkrs.utilsnew;

import com.xkrs.fireseason.FireSeasonConfigEntity;
import com.xkrs.model.bean.DataWrapper4;

import java.util.ArrayList;
import java.util.List;

public class CountyCodeUtils {

    private CountyCodeUtils() {
    }

    /**
     * 合并区划编码
     * 将获取的全部VIP用户的区划编码，进行合并，去除冗余
     */
    public static List<String> mergeCountyCodeList(List<String> countyCodeList) {
        List<String> mergedCountyCodeList = new ArrayList<>();
        for (String countyCode : countyCodeList) {
            DataWrapper4<Boolean, Boolean, List<String>, String> checkMerged = checkMerged(mergedCountyCodeList, countyCode);
            Boolean wasMerged = checkMerged.getData1();
            Boolean needReplace = checkMerged.getData2();
            List<String> oldValueList = checkMerged.getData3();
            String newValue = checkMerged.getData4();
            if (!wasMerged) {
                if (needReplace) {
                    mergedCountyCodeList.removeAll(oldValueList);
                }
                mergedCountyCodeList.add(newValue);
            }
        }
        return mergedCountyCodeList;
    }

    /**
     * 在mergedCountyCodeList中检查传入的countyCode是否被合并过了
     * DataWrapper4<Boolean, Boolean, String, String>
     * Boolean  是否被合并了
     * Boolean  如果1==true则2失效，如果1==false则2表示这个未被合并的数据是否需要替换，如果2==true则将3替换成4，如果2==false则只添加4，忽略3
     * String   如果1==false并且2==true（这个数据没有被合并，并且它需要替换添加），3是待替换的旧值
     * String   如果1==false并且2==true（这个数据没有被合并，并且它需要替换添加），4是要替换成的新值；如果1==false并且2==false（这个数据没有被合并，并且它不需要替换添加），4是要添加的
     */
    private static DataWrapper4<Boolean, Boolean, List<String>, String> checkMerged(List<String> mergedCountyCodeList, String countyCode) {
        for (String mergedCountyCode : mergedCountyCodeList) {
            try {
                String cutMergedCountyCode = FirePointCodeUtils.getFormatCutCode(mergedCountyCode);
                String cutCountyCode = FirePointCodeUtils.getFormatCutCode(countyCode);
                if (cutCountyCode.equals(cutMergedCountyCode) || cutCountyCode.startsWith(cutMergedCountyCode)) {
                    return new DataWrapper4<>(Boolean.TRUE, null, null, null);
                }
                if (cutMergedCountyCode.startsWith(cutCountyCode)) {
                    List<String> list = obtainRedundantMergedCountyCodeList(mergedCountyCodeList, cutCountyCode);
                    return new DataWrapper4<>(Boolean.FALSE, Boolean.TRUE, list, countyCode);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return new DataWrapper4<>(Boolean.FALSE, Boolean.FALSE, null, countyCode);
    }

    /**
     * 在发现了一个新的、范围更大的区划时，获取这个区划下边已经包含的、冗余的、范围小一点的区划
     */
    private static List<String> obtainRedundantMergedCountyCodeList(List<String> mergedCountyCodeList, String cutCountyCode) {
        List<String> redundantMergedCountyCodeList = new ArrayList<>();
        for (String mergedCountyCode : mergedCountyCodeList) {
            try {
                String cutMergedCountyCode = FirePointCodeUtils.getFormatCutCode(mergedCountyCode);
                if (cutMergedCountyCode.startsWith(cutCountyCode)) {
                    redundantMergedCountyCodeList.add(mergedCountyCode);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return redundantMergedCountyCodeList;
    }

    /**
     * 打印countyCodeList的信息
     */
    public static String obtainLogInfo(List<FireSeasonConfigEntity> fireSeasonConfigList, List<String> countyCodeList) {
        try {
            List<String> list = new ArrayList<>();
            for (String countyCode : countyCodeList) {
                list.add(countyCode + "(" + getCountyName(fireSeasonConfigList, countyCode) + ")");
            }
            return "[" + ListUtils.fromStringList(list, ",") + "]";
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "null";
    }

    private static String getCountyName(List<FireSeasonConfigEntity> fireSeasonConfigList, String countyCode) {
        for (FireSeasonConfigEntity fireSeasonConfig : fireSeasonConfigList) {
            if (fireSeasonConfig.getCountyCode().equals(countyCode)) {
                return fireSeasonConfig.getCountyName();
            }
        }
        return "null";
    }
}
