package com.xkrs.utilsnew;

import org.apache.hc.core5.util.TextUtils;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class DateTimeUtils {

    public final static DateTimeFormatter DATE_TIME_FORMATTER_1 = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
    public final static DateTimeFormatter DATE_TIME_FORMATTER_2 = DateTimeFormatter.ofPattern("yyyy-MM-dd");

    public static String localDateTimeToString(LocalDateTime localDateTime) {
        return DATE_TIME_FORMATTER_1.format(localDateTime);
    }

    public static LocalDateTime formatToLocalDateTime(String formattedLocalDateTime) {
        checkDateTimeFormat(formattedLocalDateTime);
        return LocalDateTime.parse(formattedLocalDateTime, DateTimeUtils.DATE_TIME_FORMATTER_1);
    }

    public static LocalDateTime stringToLocalDateTime(String localDateTimeString) {
        try {
            return LocalDateTime.parse(localDateTimeString, DATE_TIME_FORMATTER_1);
        } catch (Exception e) {
            LocalDate localDate = LocalDate.parse(localDateTimeString, DATE_TIME_FORMATTER_2);
            return LocalDateTime.of(localDate.getYear(), localDate.getMonth(), localDate.getDayOfMonth(), 0, 0, 0);
        }
    }

    /**
     * 判断时间格式 格式必须为 "yyyy-MM-dd HH:mm:ss"
     */
    public static boolean isValidDateTime(String formattedLocalDateTime) {
        int validLength = 19;
        if (TextUtils.isEmpty(formattedLocalDateTime) || formattedLocalDateTime.length() != validLength) {
            return false;
        }
        String regex = "^([1-2][0-9][0-9][0-9]-[0-1]{0,1}[0-9]-[0-3]{0,1}[0-9])\\s(20|21|22|23|[0-1]\\d):[0-5]\\d:[0-5]\\d$";
        Pattern pattern = Pattern.compile(regex);
        Matcher matcher = pattern.matcher(formattedLocalDateTime);
        return matcher.matches();
    }

    public static void checkDateTimeFormat(String formattedLocalDateTime) {
        boolean valid = isValidDateTime(formattedLocalDateTime);
        if (!valid) {
            throw new RuntimeException("日期时间格式校验失败：（" + formattedLocalDateTime + "）");
        }
    }

    public static void main(String[] args) {

        boolean validDateTime = isValidDateTime("2222-01-21 09:09:59");
        System.out.println("validDateTime = " + validDateTime);

    }

}
