package com.xkrs.utilsnew;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.xkrs.model.bean.AddressBean;
import com.xkrs.model.vo.TianDiTuGeocodeVo;
import org.apache.hc.core5.util.TextUtils;

public class FirePointAddressUtils {

    private FirePointAddressUtils() {
    }

    /**
     * 使用网络方式解析地址信息
     */
    public static AddressBean analysisWithNetwork(double longitude, double latitude) {
        AddressBean addressBean = new AddressBean();
        String geocodeStr = "";
        try {
            TianDiTuGeocodeVo geocode = TianDiTuApiUtil.geocode(longitude, latitude);
            geocodeStr = JSON.toJSONString(geocode);
            TianDiTuGeocodeVo.ResultDTO.AddressComponentDTO addressComponent = geocode.result.addressComponent;
            //填充AddressBean数据
            addressBean.setSuccess(true);
            addressBean.setMessage("");
            addressBean.setProCode(StrUtil.isEmpty(addressComponent.provinceCode) ? "" : addressComponent.provinceCode.substring(3, 9));
            addressBean.setProName(addressComponent.province);
            addressBean.setCityCode(StrUtil.isEmpty(addressComponent.cityCode) ? "" : addressComponent.cityCode.substring(3, 9));
            addressBean.setCityName(addressComponent.city);
            addressBean.setCountyCode(StrUtil.isEmpty(addressComponent.countyCode) ? "" : addressComponent.countyCode.substring(3, 9));
            addressBean.setCountyName(addressComponent.county);
            addressBean.setTownCode(StrUtil.isEmpty(addressComponent.townCode) ? "" : addressComponent.townCode.substring(3, 12));
            addressBean.setTownName(addressComponent.town);
            addressBean.setFirePointAddress(geocode.result.formattedAddress);
            return addressBean;
        } catch (Exception e) {
            e.printStackTrace();
            addressBean.setSuccess(false);
            addressBean.setMessage("longitude=" + longitude + ",latitude=" + latitude + ",geocode=" + geocodeStr + ",message=" + e.getMessage());
            return addressBean;
        }
    }

    /**
     * 检查AddressBean的合法性
     */
    private static void checkValidAddress(AddressBean addressBean) {
        if (addressBean.getProCode().length() != 6) {
            throw new RuntimeException("ProCode Error");
        }
        if (TextUtils.isEmpty(addressBean.getProName())) {
            throw new RuntimeException("ProName Error");
        }
        if (addressBean.getCityCode().length() != 6) {
            throw new RuntimeException("CityCode Error");
        }
        if (TextUtils.isEmpty(addressBean.getCityName())) {
            throw new RuntimeException("CityName Error");
        }
        if (addressBean.getCountyCode().length() != 6) {
            throw new RuntimeException("CountyCode Error");
        }
        if (TextUtils.isEmpty(addressBean.getCountyName())) {
            throw new RuntimeException("CountyName Error");
        }
        if (addressBean.getTownCode().length() != 9) {
            throw new RuntimeException("TownCode Error");
        }
        if (TextUtils.isEmpty(addressBean.getTownName())) {
            throw new RuntimeException("TownName Error");
        }
        if (TextUtils.isEmpty(addressBean.getFirePointAddress())) {
            throw new RuntimeException("FirePointAddress Error");
        }
    }

    public static void main(String[] args) {
        AddressBean addressBean = analysisWithNetwork(109.6981278, 19.8897095);
        System.out.println(addressBean.toString());
    }

}
