package com.xkrs.utilsnew;

import org.geotools.data.FeatureSource;
import org.geotools.data.Query;
import org.geotools.data.shapefile.ShapefileDataStore;
import org.geotools.data.simple.SimpleFeatureCollection;
import org.geotools.geometry.jts.ReferencedEnvelope;
import org.opengis.feature.type.AttributeDescriptor;
import org.opengis.feature.type.GeometryType;
import org.opengis.referencing.crs.CoordinateReferenceSystem;

import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.List;

public class ShpFileUtils {

    private ShpFileUtils() {
    }

    public static SimpleFeatureCollection readShpFileFeatureCollection(String shpPath) {
        File shpFile = new File(shpPath);
        SimpleFeatureCollection simpleFeatureCollection = null;
        try {
            ShapefileDataStore shapefileDataStore = new ShapefileDataStore(shpFile.toURI().toURL());
            // 设置编码,防止属性的中文字符出现乱码
            shapefileDataStore.setCharset(StandardCharsets.UTF_8);
            // 这个typeNamae不传递，默认是文件名称
            FeatureSource featuresource = shapefileDataStore.getFeatureSource(shapefileDataStore.getTypeNames()[0]);
            // 读取bbox
            ReferencedEnvelope bbox = featuresource.getBounds();
            // 读取投影
            CoordinateReferenceSystem crs = featuresource.getSchema().getCoordinateReferenceSystem();
            // 特征总数
            int count = featuresource.getCount(Query.ALL);
            // 获取当前数据的geometry类型（点、线、面）
            GeometryType geometryType = featuresource.getSchema().getGeometryDescriptor().getType();
            // 读取要素
            simpleFeatureCollection = (SimpleFeatureCollection) featuresource.getFeatures();
            // 获取当前矢量数据有哪些属性字段值
            List<AttributeDescriptor> attributes = simpleFeatureCollection.getSchema().getAttributeDescriptors();

        } catch (IOException e) {
            e.printStackTrace();
        }
        System.out.println("读取完成！");
        return simpleFeatureCollection;
    }
}
