package com.xkrs.utilsnew;

import com.xkrs.model.bean.VipTimeRangeBean;
import org.apache.hc.core5.util.TextUtils;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

public class VipTimeRangeUtils {

    private VipTimeRangeUtils() {
    }

    /**
     * 生成一个新的 VipTimeRangeJson
     */
    public static String obtainNewVipTimeRangeJson(String oldVipTimeRangeJson, LocalDateTime newLocalDateTime) {
        //没有默认VIP时间范围的情况，是新VIP用户
        if (TextUtils.isEmpty(oldVipTimeRangeJson)) {
            LocalDateTime startLocalDateTime = LocalDateTime.now();
            if (startLocalDateTime.isBefore(newLocalDateTime)) {
                VipTimeRangeBean.VipTimeRangeItemBean vipTimeRangeItemBean = new VipTimeRangeBean.VipTimeRangeItemBean();
                vipTimeRangeItemBean.setStartTime(DateTimeUtils.localDateTimeToString(startLocalDateTime));
                vipTimeRangeItemBean.setEndTime(DateTimeUtils.localDateTimeToString(newLocalDateTime));
                List<VipTimeRangeBean.VipTimeRangeItemBean> list = new ArrayList<>();
                list.add(vipTimeRangeItemBean);
                VipTimeRangeBean vipTimeRangeBean = new VipTimeRangeBean();
                vipTimeRangeBean.setList(list);
                return JsonUtils.<VipTimeRangeBean>serialize(vipTimeRangeBean);
            } else {
                throw new RuntimeException("初始化的过期时间必须晚于当前时间");
            }
        }
        //oldVipTimeRangeJson不为空，解析该用户的VIP时间范围
        VipTimeRangeBean vipTimeRangeBean = JsonUtils.deserialize(oldVipTimeRangeJson, VipTimeRangeBean.class);
        if (vipTimeRangeBean == null || vipTimeRangeBean.getList() == null || vipTimeRangeBean.getList().size() == 0) {
            throw new RuntimeException("已保存的VIP时间范围反序列化失败");
        }
        List<VipTimeRangeBean.VipTimeRangeItemBean> list = vipTimeRangeBean.getList();
        //校验数据合法性
        for (VipTimeRangeBean.VipTimeRangeItemBean vipTimeRangeItemBeanA : list) {
            for (VipTimeRangeBean.VipTimeRangeItemBean vipTimeRangeItemBeanB : list) {
                if (vipTimeRangeItemBeanA != vipTimeRangeItemBeanB) {
                    LocalDateTime startLocalDateTimeA = LocalDateTime.parse(vipTimeRangeItemBeanA.getStartTime(), DateTimeUtils.DATE_TIME_FORMATTER_1);
                    LocalDateTime endLocalDateTimeA = LocalDateTime.parse(vipTimeRangeItemBeanA.getEndTime(), DateTimeUtils.DATE_TIME_FORMATTER_1);
                    LocalDateTime startLocalDateTimeB = LocalDateTime.parse(vipTimeRangeItemBeanB.getStartTime(), DateTimeUtils.DATE_TIME_FORMATTER_1);
                    LocalDateTime endLocalDateTimeB = LocalDateTime.parse(vipTimeRangeItemBeanB.getEndTime(), DateTimeUtils.DATE_TIME_FORMATTER_1);
                    if (startLocalDateTimeB.isBefore(startLocalDateTimeA) && endLocalDateTimeB.isAfter(startLocalDateTimeA)) {
                        throw new RuntimeException("VIP开始结束时间存在交叉现象1，系统数据错误");
                    }
                    if (startLocalDateTimeB.isBefore(endLocalDateTimeA) && endLocalDateTimeB.isAfter(endLocalDateTimeA)) {
                        throw new RuntimeException("VIP开始结束时间存在交叉现象2，系统数据错误");
                    }
                }
            }
        }
        //获取结束时间最大的那个Pair
        VipTimeRangeBean.VipTimeRangeItemBean lastVipTimeRangeItemBean = list.get(0);
        for (VipTimeRangeBean.VipTimeRangeItemBean vipTimeRangeItemBean : list) {
            if (lastVipTimeRangeItemBean != vipTimeRangeItemBean) {
                LocalDateTime endLocalDateTime = LocalDateTime.parse(vipTimeRangeItemBean.getEndTime(), DateTimeUtils.DATE_TIME_FORMATTER_1);
                LocalDateTime lastEndLocalDateTime = LocalDateTime.parse(lastVipTimeRangeItemBean.getEndTime(), DateTimeUtils.DATE_TIME_FORMATTER_1);
                if (endLocalDateTime.isAfter(lastEndLocalDateTime)) {
                    lastVipTimeRangeItemBean = vipTimeRangeItemBean;
                }
            }
        }
        //获取到结束时间最大的那个Pair，用它的结束时间和当前时间做比较
        LocalDateTime lastEndLocalDateTime = LocalDateTime.parse(lastVipTimeRangeItemBean.getEndTime(), DateTimeUtils.DATE_TIME_FORMATTER_1);
        if (lastEndLocalDateTime.isBefore(LocalDateTime.now())) {
            //最大结束时间早于当前时间，说明该用户已过期，是普通用户。
            //调整普通用户过期时间的方案是添加新的Pair，开始时间是当前时间，结束时间是传入的时间
            LocalDateTime startLocalDateTime = LocalDateTime.now();
            if (startLocalDateTime.isBefore(newLocalDateTime)) {
                VipTimeRangeBean.VipTimeRangeItemBean vipTimeRangeItemBean = new VipTimeRangeBean.VipTimeRangeItemBean();
                vipTimeRangeItemBean.setStartTime(DateTimeUtils.localDateTimeToString(startLocalDateTime));
                vipTimeRangeItemBean.setEndTime(DateTimeUtils.localDateTimeToString(newLocalDateTime));
                list.add(vipTimeRangeItemBean);
                return JsonUtils.<VipTimeRangeBean>serialize(vipTimeRangeBean);
            } else {
                throw new RuntimeException("为断更用户添加的过期时间必须晚于当前时间");
            }
        } else {
            //最大结束时间晚于当前时间，说明该用户未过期，是VIP用户。
            //调整VIP用户过期时间的方案是调整结束时间为传入的时间
            LocalDateTime startLocalDateTime = LocalDateTime.parse(lastVipTimeRangeItemBean.getStartTime(), DateTimeUtils.DATE_TIME_FORMATTER_1);
            if (startLocalDateTime.isBefore(newLocalDateTime)) {
                lastVipTimeRangeItemBean.setEndTime(DateTimeUtils.localDateTimeToString(newLocalDateTime));
                return JsonUtils.<VipTimeRangeBean>serialize(vipTimeRangeBean);
            } else {
                throw new RuntimeException("设置的过期时间必须晚于当次的VIP开始时间");
            }
        }
    }

    public static void main(String[] args) {
        VipTimeRangeBean.VipTimeRangeItemBean vipTimeRangeItemBean1 = new VipTimeRangeBean.VipTimeRangeItemBean();
        vipTimeRangeItemBean1.setStartTime("2022-06-27 11:15:00");
        vipTimeRangeItemBean1.setEndTime("2022-07-07 18:20:00");
        VipTimeRangeBean.VipTimeRangeItemBean vipTimeRangeItemBean2 = new VipTimeRangeBean.VipTimeRangeItemBean();
        vipTimeRangeItemBean2.setStartTime("2022-07-09 18:20:00");
        vipTimeRangeItemBean2.setEndTime("2022-07-20 18:20:00");
        List<VipTimeRangeBean.VipTimeRangeItemBean> list = new ArrayList<>();
        list.add(vipTimeRangeItemBean1);
        list.add(vipTimeRangeItemBean2);
        VipTimeRangeBean vipTimeRangeBean = new VipTimeRangeBean();
        vipTimeRangeBean.setList(list);
        String serialize = JsonUtils.<VipTimeRangeBean>serialize(vipTimeRangeBean);
        System.out.println("serialize = " + serialize);
    }

}
