package com.xkrs.utilsold;

import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.time.format.DateTimeFormatter;

/**
 * 日期时间工具
 */
public class DateTimeUtil {

    public final static String COMMON_PATTERN_DATETIME = "yyyy-MM-dd HH:mm:ss";
    public final static String COMMON_PATTERN_DATE = "yyyy-MM-dd";
    public final static DateTimeFormatter COMMON_FORMATTER_DATETIME = DateTimeFormatter.ofPattern(COMMON_PATTERN_DATETIME);
    public final static DateTimeFormatter COMMON_FORMATTER_DATE = DateTimeFormatter.ofPattern(COMMON_PATTERN_DATE);
    public final static ZoneOffset DEFAULT_ZONE_OFFSET = ZoneOffset.of("+8");

    /**
     * LocalDateTime转字符串
     */
    public static String dateTimeToString(LocalDateTime dateTime) {
        assert dateTime != null;
        return COMMON_FORMATTER_DATETIME.format(dateTime);
    }

    /**
     * 字符串转LocalDateTime
     */
    public static LocalDateTime stringToDateTime(String date) {
        LocalDateTime parse = null;
        try {
            parse = LocalDateTime.parse(date, COMMON_FORMATTER_DATETIME);
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (parse == null) {
            return LocalDateTime.parse(date, COMMON_FORMATTER_DATE);
        }
        return parse;
    }

    /**
     * 字符串转LocalDateTime（格式化）
     */
    public static LocalDateTime stringToDateTimeFormatter(String dateStr, DateTimeFormatter formatter) {
        assert dateStr != null;
        return LocalDateTime.parse(dateStr, formatter);
    }


    /**
     * 时间戳转日期
     */
    public static LocalDateTime timeMillisToDate(long timeMillis) {
        Instant instant = Instant.ofEpochMilli(timeMillis);
        return LocalDateTime.ofInstant(instant, DEFAULT_ZONE_OFFSET);
    }

    /**
     * 时间戳转时间再转字符串
     */
    public static String timeMillisToString(long timeMillis) {
        LocalDateTime localDateTime = LocalDateTime.ofEpochSecond(timeMillis, 0, ZoneOffset.ofHours(8));
        return COMMON_FORMATTER_DATETIME.format(localDateTime);
    }

    /**
     * 获取当前时间 yyyy-MM-ss hh:mm:ss
     */
    public static String getNowTimeStr() {
        return COMMON_FORMATTER_DATETIME.format(LocalDateTime.now());
    }

}
